/**
 BorderPanel.java
 
 *  By Wayne Conrad <wconrad@pcsiaz.com>, March 1998.
 *  Java 1.1
 *  Use at your own risk!
 
 *  This file is part of DeepMatrixIP9.
 *
 *  DeepMatrixIP9 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DeepMatrix IP9 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.deck;

import java.awt.*;

/**
 * This class is a panel with a border.
 */

public class BorderPanel extends Panel {

  // Border styles.

  public final static int NONE       = 0;
  public final static int PLAIN      = 1;
  public final static int ETCHED_IN  = 2;
  public final static int ETCHED_OUT = 3;
  public final static int RAISED     = 4;
  public final static int LOWERED    = 5;

  // The border's style.

  protected int style = PLAIN;

  // The inner margin, which is the space between the drawn border and the
  // panel's contents.

  int innerMargin = 3;


  // The outer margin, which is the space between the panel's outer edge and
  // the drawn border.

  int outerMargin = 3;

/**
 * Constructor using the default style (PLAIN).
 */
  public BorderPanel() {}

/**
 * Constructor taking the border style
 */
  public BorderPanel (int style) {this.style = style;}

/**
 * Return the current style
 */
  public int getStyle() {return style;}


/**
 * Change the border's style.  This causes the component to be redrawn.
 */
  public void setStyle (int style) {
    this.style = style;
    repaint();
  }

/**
 * Return the inner margin.  This is the space between the drawn border and
 * the panel's contents.
 */
  public int getInnerMargin() {
    return innerMargin;
  }

/**
 * Set the inner margin.  This is the space between the drawn border and
 * the panel's contents.
 */
  public void setInnerMargin (int innerMargin) {
    this.innerMargin = innerMargin;
    repaint();
  }

/**
 * Return the outer margin.  This is the space between the outer edge of
 * the panel and the drawn border.
 */
  public int getOuterMargin() {
    return outerMargin;
  }

/**
 * Set the outer margin.  This is the space between the outer edge of the
 * panel and the drawn border.
 */
  public void setOuterMargin (int outerMargin) {
    this.outerMargin = outerMargin;
    repaint();
  }

/**
 * new school
 */
  public Insets getInsets() {
    int width;
    switch (style) {
      case NONE : width = 0; break;
      case PLAIN: width = 1; break;
      default   : width = 2;
    }
    width += innerMargin;
    width += outerMargin;
    return new Insets (width, width, width, width);
  }

/**
 * old school
 */
 public Insets insets() {
   return getInsets();
 }

/**
 * (Override) Paint the border.
 */
  public void paint (Graphics g) {
    switch (style) {
      case NONE      : drawNone(); break;
      case ETCHED_IN : drawEtched (g, darkerColor(), brighterColor() ); break;
      case ETCHED_OUT: drawEtched (g, brighterColor(), darkerColor() ); break;
      case RAISED    : draw3d (g, brighterColor(), darkerColor() ); break;
      case LOWERED   : draw3d (g, darkerColor(), brighterColor() ); break;
      case PLAIN:
      default:
        drawPlain (g);
    }
  }

/**
 * Draw an empty border.
 */
  protected void drawNone() {}

/**
 * Draw a plain border.
 */
  protected void drawPlain (Graphics g) {
    Dimension size = size();
    int x = outerMargin;
    int y = outerMargin;
    int width = size.width - 2 * outerMargin - 1;
    int height = size.height - 2 * outerMargin - 1;
    g.drawRect (x, y, width, height);
  }

/**
 * Draw an etched border.
 */
  protected void drawEtched(Graphics g, Color upperLeftColor, Color lowerRightColor) {
    Dimension size = size();
    g.setColor (upperLeftColor);
    int x = outerMargin;
    int y = outerMargin;
    int width = size.width - 2 * outerMargin - 2;
    int height = size.height - 2 * outerMargin - 2;
    g.drawRect (x, y, width, height);
    g.setColor (lowerRightColor);
    g.drawRect (x + 1, y + 1, width, height);
  }

/**
 * Draw a 3d border.
 */
  protected void draw3d(Graphics g, Color upperLeftColor, Color lowerRightColor) {
    for (int i = 0; i < 2; i++) {
      Dimension size = size();
      int x1 = i + outerMargin;
      int y1 = i + outerMargin;
      int x2 = size.width - i - outerMargin - 1;
      int y2 = size.height - i - outerMargin - 1;
      g.setColor (upperLeftColor);
      g.drawLine (x1, y1, x2, y1);
      g.drawLine (x1, y1, x1, y2);
      g.setColor (lowerRightColor);
      g.drawLine (x2, y1, x2, y2);
      g.drawLine (x1, y2, x2, y2);
    }
  }

/**
 * Return the brighter color to use.  This doesn't work right if the
 * background color is white.  However, I have found no combination of
 * colors that complete the 3d illusion when the background is white, so
 * it's not worth trying to work around.
 */
  protected Color brighterColor() {
    return getBackground().brighter();
  }

/**
 * Return the darker color to use.  This doesn't work right if the
 * background color is white.  However, I have found no combination of
 * colors that complete the 3d illusion when the background is white, so
 * it's not worth trying to work around.
 */
  protected Color darkerColor() {
    return getBackground().darker();
  }
}
