/*
  Network.java


  Copyright (C) 1998 Geometrek	and 2009 by R. Russell Kinter
 
 *  This file is part of DeepMatrixIP9.
 *
 *  DeepMatrixIP9 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DeepMatrix IP9 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.deck;

import matrix.RoomControl;
import matrix.vrml.Field;
import java.io.*;
import java.util.*;

/**
 * The Network represents the API to work with the network protocols of DeepMatrix
 *
 * @version 1.2
 */
public interface Network {

    // implement the functionality of the control protocol ( TCP )

    /**
     * open a connection to the Matrix Server.
     * @param hostname hostname of the server to connect to
     * @param port     port number TCP port of server
     * @param name     users name
     * @param passwd   password. not used currently
     * @param room     room to initialy start in. if NULL the servers default room is used.
     * @param ARG   New 11_12_05 tells the server to hold off putting the client in the selected room until the client has 
     * run finalization and garbage collection.
     */
    public void connect( String hostname, int port, String name, String passwd, String room, String ARG ) throws IOException;

    public void startRTThread();


    /**
     * close connection to server and clears Consumers.
     */
    public void disconnect( String tf );

    /**
     * returns true if Network is connected to a server, otherwise false.
     */
    public boolean isConnected();

    /**
     * requires update on networkState nodes
     */
    public void getStates() throws IOException, InterruptedException;

    /**
     * builds and sends a chat message to the server.
     *
     * @param to name of client to send the chat message to. If NULL then it is send to all.
     * @param line content of chat message.
     */
    public void sendChatMessage( String to, String line ) throws IOException;

    
	public void askAdminControl( String name, String password ) throws IOException, InterruptedException;

	public void sendAdminControl( int icom, String name, String command ) throws IOException, InterruptedException;

	 /**
     *  New 11-12-05 sendHeartBeat polls server every 7 seconds.
     */

    public void sendHeartBeat( String name ) throws IOException, InterruptedException;

    /**
     *  New 11-12-05 Trying to coordinate the start of the MsgBuffer 
     *  of the inner Connection$MessageBuffer class before starting the Viewer thread.
     *  This will be done by RoomLoader calling gateFired2 method of Beta during the intitial loading. 
     */

    public void startMsgBuffer();

	public void sendIgnore( String target, String tf ) throws IOException, InterruptedException;

	public void sendControlEvent( String name, String [] owner ) throws IOException, InterruptedException;

	public void sendControlRoom( String name, String [] owner ) throws IOException, InterruptedException;

	 public void sendObjectRequest( String name, String state ) throws IOException, InterruptedException;
    /**
     * tries to add an Object to the room. This sends an OBJADD message to the server.
     *
     * @param name object name, should be unique in the room
     * @param url  URL of a VRML defining the object
     */
    public void addSharedObject( String name, String url ) throws IOException, InterruptedException;

    public void sendRoomControl( RoomControl rc ) throws IOException, InterruptedException;

    /**
     * sends an OBJDEL message to the server to delete an object.
     *
     * @param name the objects name
     */
    public void deleteSharedObject( String name ) throws IOException, InterruptedException;

    /**
     * updates the states of a certain object. to update the world
     * use name "world" right now.
     * 
     * @param name the objects name
     * @param obj true: SharedObject update with different protocol UDPOBJ
     */

    public void update( String name, boolean obj ) throws IOException, InterruptedException ;    

    /**
     * switches to another room by sending a ROOM message to the server.
     *
     * @param name the rooms name
     * @param arg New 11_12_05 tells the server the client is ready to enter the new room.
     */

    public void switchRoom( String name, String arg ) throws IOException, InterruptedException ;

    /**
     * sends a WHOIS message to the server to request an updated list of users.
     */
    public void whois() throws IOException, InterruptedException ;

    // adding / removing NetworkListener

    public void addNetworkListener( NetworkListener listener );

    public void removeNetworkListener( NetworkListener listener );

    // real time message protocol

    public void setNavChange( String [] types );


    public void setSpeedChange ( float ft );

    // These are for managing Consumers
    // Consumers receive real time messages

    public void addConsumer( RealTimeConsumer consumer );

    public RealTimeConsumer getConsumer( String name );
  
    public void handleUserPosition( float [] flt );

    public void removeConsumer( String name );

    public Enumeration getConsumers();

    public void clearConsumers();

    //public String getUserName();

    /**
     * sends a RealTimeMessage to the server.
     * This is used by the classes handling updates in the world as Route and UserAvatar.
     *
     * @param stor signaling persistent storage NONE or while room populated or forever.
     * @param tag the fields tag ( either networkState tag or object name )
     * @param data data to be updated
     * @param echo flag signaling whether a copy should be send back to the sender
     * @see matrix.RealTimeMessage
     */
    public void sendRealTime( int stor, String tag, Field data, boolean echo ) throws IOException;

	/**
     * sends a RealTimeSave to the server to be written to file for persistent storage
     *
     * @param tag the fields tag ( either networkState tag or object name )
     * @param data data to be updated always an MFFloat field (translation & rotation) at the moment
     * @param echo flag useless at the moment always set to false
     * @see matrix.RealTimeMessage
     */

	public void sendRealTimeSave( String tag, Field data, Field state ) throws IOException, InterruptedException;
}
